# --
# Kernel/Config.pm - Config file for OTRS kernel
# Copyright (C) 2001-2010 OTRS AG, http://otrs.org/
# --
# $Id: Config.pm.dist,v 1.23 2010/01/13 22:25:00 martin Exp $
# --
# This software comes with ABSOLUTELY NO WARRANTY. For details, see
# the enclosed file COPYING for license information (AGPL). If you
# did not receive this file, see http://www.gnu.org/licenses/agpl.txt.
# --
#  Note:
#
#  -->> OTRS does have a lot of config settings. For more settings
#       (Notifications, Ticket::ViewAccelerator, Ticket::NumberGenerator,
#       LDAP, PostMaster, Session, Preferences, ...) see
#       Kernel/Config/Defaults.pm and copy your wanted lines into "this"
#       config file. This file will not be changed on update!
#
# --

package Kernel::Config;

sub Load {
    my $Self = shift;
    # ---------------------------------------------------- #
    # ---------------------------------------------------- #
    #                                                      #
    #         Start of your own config options!!!          #
    #                                                      #
    # ---------------------------------------------------- #
    # ---------------------------------------------------- #

    # ---------------------------------------------------- #
    # database settings                                    #
    # ---------------------------------------------------- #
    # DatabaseHost
    # (The database host.)
    $Self->{'DatabaseHost'} = 'localhost';
    # Database
    # (The database name.)
    $Self->{'Database'} = 'otrs';
    # DatabaseUser
    # (The database user.)
    $Self->{'DatabaseUser'} = 'otrs';
    # DatabasePw
    # (The password of database user. You also can use bin/otrs.CryptPassword.pl
    # for crypted passwords.)
    $Self->{'DatabasePw'} = 'password';
    # DatabaseDSN
    # (The database DSN for MySQL ==> more: "man DBD::mysql")
    $Self->{DatabaseDSN} = "DBI:mysql:database=$Self->{Database};host=$Self->{DatabaseHost};";

    # (The database DSN for PostgreSQL ==> more: "man DBD::Pg")
    # if you want to use a local socket connection
#    $Self->{DatabaseDSN} = "DBI:Pg:dbname=$Self->{Database};";
    # if you want to use a tcpip connection
#    $Self->{DatabaseDSN} = "DBI:Pg:dbname=$Self->{Database};host=$Self->{DatabaseHost};";

    # ---------------------------------------------------- #
    # fs root directory
    # ---------------------------------------------------- #
    $Self->{Home} = '/opt/otrs';

    # ---------------------------------------------------- #
    # insert your own config settings "here"               #
    # config settings taken from Kernel/Config/Defaults.pm #
    # ---------------------------------------------------- #
    # $Self->{SessionUseCookie} = 0;
    # $Self->{CheckMXRecord} = 0;

    # ---------------------------------------------------- #

    # ---------------------------------------------------- #
    # data inserted by installer                           #
    # ---------------------------------------------------- #
    # $DIBI$
    $Self->{'DefaultCharset'} = 'utf-8';

	# DV stuff
	my $c_host = 'host.realm.hr';
	$Self->{'Organization'} = 'Organization';
    $Self->{'LogModule::LogFile'} = '/var/log/otrs/otrs.log';
    $Self->{'LogModule'} = 'Kernel::System::Log::File';
    $Self->{'FQDN'} = $c_host;
    $Self->{'DefaultLanguage'} = 'en';
    $Self->{'AdminEmail'} = 'root@realm.hr';
	$Self->{'CustomerGroupSupport'} =  '1';
	$Self->{'CustomerGroupAlwaysGroups'} =  [
  			'users'
	];


 #
    # AgentUser authentication
    #
    ###########################
    $Self->{'AuthModule'} = 'Kernel::System::Auth::simpleSAMLphpAgentAuth';
    # simpleSAMLphp agent authentication module
    $Self->{'LoginURL'} = 'http://'.$c_host.'/simplesaml/otrs/agent_auth.php';
    # simpleSAMLphp SP logout URL
    $Self->{'LogoutURL'} = 'http://'.$c_host.'/simplesaml/otrs/logout.html';


    #
    # CustomerUser authentication
    #
    ##############################
    #$Self->{'Customer::AuthModule'} = 'Kernel::System::Auth::simpleSAMLphpCustomerAuth';
    $Self->{'Customer::AuthModule'} = 'Kernel::System::CustomerAuth::simpleSAMLphpCustomerAuth';
    # simpleSAMLphp customer authentication module
    $Self->{'CustomerPanelLoginURL'} = 'http://'.$c_host.'/simplesaml/otrs/customer_auth.php';
    # simpleSAMLphp SP logout URL
    $Self->{'CustomerPanelLogoutURL'} = 'http://'.$c_host.'/simplesaml/otrs/logout.html';

 #
    # CustomerUser data mapping
    #
    ############################
    $Self->{CustomerUser} = {
        Module => 'Kernel::System::CustomerUser::simpleSAMLphpCustomerData',
        Name => 'Database Backend',
        Params => {
            Table => 'customer_user',
        },
        CustomerKey => 'login',
        CustomerID => 'customer_id',
        CustomerValid => 'valid_id',
        CustomerUserListFields => ['first_name', 'last_name', 'email'],
        CustomerUserSearchFields => ['login', 'first_name', 'last_name', 'customer_id'],
        CustomerUserSearchPrefix => '',
        CustomerUserSearchSuffix => '*',
        CustomerUserSearchListLimit => 500,
        CustomerUserPostMasterSearchFields => ['email'],
        CustomerUserNameFields => ['salutation', 'first_name', 'last_name'],
        CustomerUserEmailUniqCheck => 1,
		 Map => [
            # note: Login, Email and CustomerID needed!
            # var, frontend, storage, shown (1=always,2=lite), required, storage-type, http-link, readonly
            [ 'UserFirstname',  'Firstname',  'first_name',  1, 1, 'var', '', 0 ],
            [ 'UserLastname',   'Lastname',   'last_name',   1, 1, 'var', '', 0 ],
            [ 'UserLogin',      'Username',   'login',       1, 1, 'var', '', 0 ],
            [ 'UserEmail',      'Email',      'email',       0, 1, 'var', '', 0 ],
            [ 'UserCustomerID', 'CustomerID', 'customer_id', 0, 1, 'var', '', 0 ],
            [ 'UserComment',    'Comment',    'comments',    1, 0, 'var', '', 0 ],
            [ 'ValidID',        'Valid',      'valid_id',    0, 1, 'int', '', 0 ],
        ],
    };


    #
    # simpleSAMLphp integration
    # (added by ddzeko@srce.hr and dvoncina@srce.hr)
    #
	#################################################
    $Self->{'simpleSAML::SessionSavePath'} = '/var/run/samlsessions';
    $Self->{'simpleSAML::SessionCookieName'} = 'PHPSESSID';

    # ---------------------------------------------------- #
    # ---------------------------------------------------- #
    #                                                      #
    #           End of your own config options!!!          #
    #                                                      #
    # ---------------------------------------------------- #
    # ---------------------------------------------------- #
}

# ---------------------------------------------------- #
# needed system stuff (don't edit this)                #
# ---------------------------------------------------- #
use strict;
use warnings;

use vars qw(@ISA $VERSION);
$VERSION = qw($Revision: 1.23 $)[1];

use Kernel::Config::Defaults;
push (@ISA, 'Kernel::Config::Defaults');

# -----------------------------------------------------#

1;
