# --
# Kernel/System/Auth/simpleSAMLphpAgentAuth.pm - provides agent
# authentication based on simpleSAMLphp SSO session
# --
# Copyright (C) 2008 Dubravko Voncina & Damir Dzeko
# --
# $Id: simpleSAMLphpAgentAuth.pm, v 0.9 2008/07/01
# --

package Kernel::System::Auth::simpleSAMLphpAgentAuth;

use strict;

use PHP::Session; 
use vars qw($VERSION);
$VERSION = '$Revision: 1.5 $';
$VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;

sub new {
    my $Type = shift;
    my %Param = @_;

    # allocate new hash for object
    my $Self = {};
    bless ($Self, $Type);

    # check needed objects
    foreach (qw(LogObject ConfigObject DBObject)) {
        $Self->{$_} = $Param{$_} || die "No $_!";
    }

    # Debug 0=off 1=on
    $Self->{Debug} = 0;

    return $Self;
}

sub GetOption {
    my $Self = shift;
    my %Param = @_;
    # check needed stuff
    if (!$Param{What}) {
        $Self->{LogObject}->Log(Priority => 'error', Message => "Need What!");
        return;
    }
    # module options
    my %Option = (
        PreAuth => 1,
    );
    # return option
    return $Option{$Param{What}};
}

sub Auth {
    my $Self = shift;
    my %Param = @_;
    my $RemoteAddr = $ENV{REMOTE_ADDR} || 'Got no REMOTE_ADDR env!';
    # get params

    my $sid='';

    my $rcvd_cookies = $ENV{'HTTP_COOKIE'};
    my @cookie_keyvals = split /;\s*/, $rcvd_cookies;
    my %cookies;
    
    my ($save_path, $sess_cookie);
    $save_path = $Self->{ConfigObject}->{'simpleSAML::SessionSavePath'}
       || '/var/run/samlsessions';
    $save_path .= '/' unless ($save_path =~ m!/$!);
    $sess_cookie = $Self->{ConfigObject}->{'simpleSAML::SessionCookieName'}
       || 'PHPSESSID';
       
    sub url_decode {
      my $string = shift;
      $string =~ tr/+/ /;
      $string =~ s/%([\da-fA-F]{2})/chr (hex ($1))/eg;
      return $string;
    }                      

    my ($key, $value, $keyval);
    foreach $keyval (@cookie_keyvals) {
      ($key, $value) = split (/=/, $keyval, 2);
      $value = '' unless defined $value;
      $key   = url_decode($key);
      $value = url_decode($value);
      if ( defined ($cookies{$key}) ) {
        $cookies{$key} = [$cookies{$key}] 
          unless ( ref $cookies{$key} );
        push (@{ $cookies{$key} }, $value);
      } else {
        $cookies{$key} = $value;
      }
    }

    if (exists($cookies{$sess_cookie})) {                                                                                                                                                            
      if( -e $save_path.'sess_'.$cookies{$sess_cookie} ){
	$sid = $cookies{$sess_cookie};
      }
    }

    my $User = '';

    if( $sid ne "" ){
    	my $session = PHP::Session->new( $sid, { save_path => $save_path } );
    	$User = $session->get('simplesamlphp_otrs_user_id');
    }

    if ($User) {
        my $Replace = $Self->{ConfigObject}->Get('AuthModule::HTTPBasicAuth::Replace');
        if ($Replace) {
            $User =~ s/^\Q$Replace\E//;
        }
        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message => "User: $User authentication ok (REMOTE_ADDR: $RemoteAddr).",
        );
        return $User;
    }
    else {
        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message => "User: No \$ENV{REMOTE_USER} !(REMOTE_ADDR: $RemoteAddr).",
        );
        return;
    }
}


1;
