# Kernel/System/Auth/simpleSAMLphpCustomerAuth.pm - provides customer
# authentication based on simpleSAMLphp SSO session
# --
# Copyright (C) 2008 Dubravko Voncina & Damir Dzeko
# --
# $Id: simpleSAMLphpCustomerAuth.pm, v 0.9 2008/01/07 08:00:00
# --

package Kernel::System::CustomerAuth::simpleSAMLphpCustomerAuth;

use strict;

use PHP::Session;
use vars qw($VERSION);
$VERSION = '$Revision: 1.5 $';
$VERSION =~ s/^\$.*:\W(.*)\W.+?$/$1/;

sub new {
    my $Type = shift;
    my %Param = @_;

    # allocate new hash for object
    my $Self = {};
    bless ($Self, $Type);

    # check needed objects
    foreach (qw(LogObject ConfigObject DBObject)) {
        $Self->{$_} = $Param{$_} || die "No $_!";
    }

    # Debug 0=off 1=on
    $Self->{Debug} = 0;
    return $Self;
}

sub GetOption {
    my $Self = shift;
    my %Param = @_;
    # check needed stuff
    if (!$Param{What}) {
        $Self->{LogObject}->Log(Priority => 'error', Message => "Need What!");
        return;
    }
    # module options
    my %Option = (
        PreAuth => 1,
    );
    # return option
    return $Option{$Param{What}};
}

sub Auth {
    my $Self = shift;
    my %Param = @_;
    my $RemoteAddr = $ENV{REMOTE_ADDR} || 'Got no REMOTE_ADDR env!';
    # get params

    my $sid='';

    if( $ENV{'HTTP_COOKIE'} ){
        my $rcvd_cookies = $ENV{'HTTP_COOKIE'};
        my @cookies = split( /;/, $rcvd_cookies );

        foreach my $cookie( @cookies ){
		if ( index( $cookie, "PHPSESSID=" ) > -1 ){

#print "Content-type:text/html\n\n";
#print $cookie."<br>";
#print substr( $cookie, index( $cookie, "PHPSESSID=" )+10  );
#exit();
	    		# 15.05.2008. modified by Dubravko Voncina
	    		# verify if session data file exists (we assume that session data files
	    		# are stored in /var/run/samlsessions/ folder)
			# (this is tricky, it seems that sometimes you have to eliminate first 10 characters
			# and sometimes you have to eliminate first 11 characters from the $cookie variable
			# to get the session id, it may depend on php configuration)
	    		# if( -e '/var/run/samlsessions/sess_'.substr( $cookie, 10 ) ){
			#	$sid = substr( $cookie, 10 );
	    		#}
	    		if( -e '/var/run/samlsessions/sess_'.substr( $cookie, index( $cookie, "PHPSESSID=" ) + 10 ) ){
				$sid = substr( $cookie, index( $cookie, "PHPSESSID=" ) + 10 );
	    		}
        	}
        }
    }

    my $User = '';
    my $UserEmail = '';
    my $UserFirstname = '';
    my $UserLastname = '';

#print "Content-type:text/html\n\n";
#print $ENV{'HTTP_COOKIE'};
#print "<br>";
#print "sid: $sid";
#exit();
    if( $sid ne "" ){
	# get session data stored by simpleSAMLphp in session file
	my $session = PHP::Session->new( $sid, { save_path => '/var/run/samlsessions/' } );
    	$User = $session->get('simplesamlphp_otrs_user_id');
    	$UserEmail = $session->get('simplesamlphp_otrs_user_mail');
    	$UserFirstname = $session->get('simplesamlphp_otrs_user_givenname');
    	$UserLastname = $session->get('simplesamlphp_otrs_user_sn');
    }

    if ($User ne "") {
        my $Replace = $Self->{ConfigObject}->Get('AuthModule::HTTPBasicAuth::Replace');
        if ($Replace) {
            $User =~ s/^\Q$Replace\E//;
        }

	# enter the most recent customer user data into customer_user table
	my ($Sec, $Min, $Hour, $Day, $Month, $Year) = localtime(time());
    	$Year += 1900;
    	$Month += 1;
    	my $CurrentTime = $Year.'-'.$Month.'-'.$Day.' '.$Hour.':'.$Min.':'.$Sec;
    	my $SQL = "SELECT COUNT(*) AS customer_user_count FROM customer_user WHERE login='".$User."'";
    	$Self->{DBObject} -> Prepare(SQL => $SQL);
    	my @Row = $Self->{DBObject}->FetchrowArray();

    	if( $Row[0] > 0){

    		# update customer_user table
        	$SQL = "UPDATE customer_user SET email='".$UserEmail."', first_name='".$UserFirstname."', last_name='".$UserLastname."', change_time='".$CurrentTime."' WHERE login='".$User."'";

			if ($Self->{DBObject}->Do(SQL => $SQL)) {
            	$Self->{LogObject}->Log(
                	Priority => 'notice',
                	Message => "CustomerUser: '$User' updated successfully!",
            	);
        	}
    	}
	else{
		# insert user data into customer_user table
        	$SQL = "INSERT INTO customer_user (login, customer_id, email, first_name, last_name, valid_id, create_time, change_time) VALUES ('".$User."', '".$User."', '".$UserEmail."', '".$UserFirstname."', '".$UserLastname."', 1, '".$CurrentTime."', '".$CurrentTime."')";
		$Self->{DBObject}->Do(SQL => $SQL);
        	$SQL = "INSERT INTO customer_preferences (user_id, preferences_key, preferences_value) VALUES ('".$User."', 'UserShowTickets', 25)";
		$Self->{DBObject}->Do(SQL => $SQL);
        	$SQL = "INSERT INTO customer_preferences (user_id, preferences_key, preferences_value) VALUES ('".$User."', 'UserRefreshTime', 2)";
		$Self->{DBObject}->Do(SQL => $SQL);
        	$SQL = "INSERT INTO customer_preferences (user_id, preferences_key, preferences_value) VALUES ('".$User."', 'UserLanguage', 'en')";
		$Self->{DBObject}->Do(SQL => $SQL);
        	$SQL = "INSERT INTO customer_preferences (user_id, preferences_key, preferences_value) VALUES ('".$User."', 'UserShowClosedTickets', 1)";
		$Self->{DBObject}->Do(SQL => $SQL);
        	$SQL = "INSERT INTO customer_preferences (user_id, preferences_key, preferences_value) VALUES ('".$User."', 'UserTheme', 'Standard')";
		$Self->{DBObject}->Do(SQL => $SQL);

        	# log notice
            	$Self->{LogObject}->Log(
                	Priority => 'notice',
                	Message => "CustomerUser: '$User' created.",
            	);
    	}


        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message => "User: $User authentication ok (REMOTE_ADDR: $RemoteAddr).",
        );
        return $User;
    }
    else {
        $Self->{LogObject}->Log(
            Priority => 'notice',
            Message => "User: No \$ENV{REMOTE_USER} !(REMOTE_ADDR: $RemoteAddr).",
        );
        return;
    }
}


1;
