<?php

require_once('../../lib/_autoload.php');
require_once( "lib/config.php" );
require_once( "lib/db_mysql.php" );
$as = new SimpleSAML_Auth_Simple('default-sp');

session_start();
session_register( "simplesamlphp_otrs_user_id" );
session_register( "simplesamlphp_otrs_user_mail" );
session_register( "simplesamlphp_otrs_user_givenname" );
session_register( "simplesamlphp_otrs_user_sn" );

/* Setting initial values of user attributes required by OTRS (user_id, mail, givenname, sn): */
$_SESSION['simplesamlphp_otrs_user_id'] = NULL;
$_SESSION['simplesamlphp_otrs_user_mail'] = NULL;
$_SESSION['simplesamlphp_otrs_user_givenname'] = NULL;
$_SESSION['simplesamlphp_otrs_user_sn'] = NULL;
$_SESSION['simplesamlphp_otrs_user_homeorg'] = NULL;

/* Load simpleSAMLphp, configuration and metadata and check if valid local session exists.. */
$as->requireAuth();

/* Get required attributes from simpleSAMLphp SSO session: */
$attributes = $as->getAttributes();
if( isset($attributes['hrEduPersonUniqueID']))
	$_SESSION['simplesamlphp_otrs_user_id'] = $attributes['hrEduPersonUniqueID'][0];
if( isset($attributes['mail']))
	$_SESSION['simplesamlphp_otrs_user_mail'] = $attributes['mail'][0];
if( isset($attributes['givenName']))
	$_SESSION['simplesamlphp_otrs_user_givenname'] = $attributes['givenName'][0];
if( isset($attributes['sn']))
	$_SESSION['simplesamlphp_otrs_user_sn'] = $attributes['sn'][0];
if( isset($attributes['hrEduPersonHomeOrg']))
    $_SESSION['simplesamlphp_otrs_user_homeorg'] = $attributes['hrEduPersonHomeOrg'][0];

/* Allow only users from realms in lib/config.php@ALLOWED_REALMS*/
function realm_is_ok($realm_in){
	$ok = 0;						// Default is: NOT OK
	$a_realms = array();
	if(ALLOWED_REALMS == '*'){		// If all realms are allowed '*'
		$ok = 99999;
	}
	else {							// Specific realms are allowed
		$a_realms = split(',', ALLOWED_REALMS);
		foreach ($a_realms as $realm){
			if($realm == $realm_in){
				$ok++;
			}
		}
	}
	return $ok;
}

/* Verify if all required attributes are provided: */
if( $_SESSION['simplesamlphp_otrs_user_id'] == NULL ||
    $_SESSION['simplesamlphp_otrs_user_mail'] == NULL ||
    $_SESSION['simplesamlphp_otrs_user_givenname'] == NULL ||
    $_SESSION['simplesamlphp_otrs_user_sn'] == NULL ||
	realm_is_ok($_SESSION['simplesamlphp_otrs_user_homeorg']) == 0){
	echo ' 	<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
               	<html>
               	<head>
              		<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                        <title>OTRS</title>
			<style type="text/css">
			<!--
				.header { color:#efefef;background-color:#ffffff;font-weight:bold;font-size:16px;font-family:Geneva,Helvetica,Arial,sans-serif;border-style: none none solid none; border-color:#bcd5e4;}
				.footer { color:#193153;background-color:#bcd5e4;font-weight:bold;font-size:10px;font-family:Geneva,Helvetica,Arial,sans-serif; }
			-->
			</style>
                </head>
                <body>
		<table width="850" border="0">
			<tr height="60">
				<td class="header" colspan="3">&nbsp;</td>
			</tr>';
	/* Realms are OK but other attribues are missing */
	if(realm_is_ok($attributes['hrEduPersonHomeOrg'][0]) > 0){
		echo '
			<tr height="160">
                <td width="20">&nbsp;</td>
				<td align="left" valign="middle" colspan="2">
					Sorry, but it seems that your IdP did not provide all required attributes.<br>
                        		Following attributes must be provided:
                        		<ul>
                        			<li>hrEduPersonUniqueID</li>
                        			<li>sn</li>
                        			<li>givenName</li>                        			
                        			<li>mail</li>
									<li>hrEduPersonHomeOrg</li>
                        		</ul>
				</td>
			</tr>
			<tr height="150">
				<td width="20">&nbsp;</td>
				<td align="left" valign="middle" colspan="2">
					Your IdP provided following attribute values:<br>
                        		<ul>
                        			<li>hrEduPersonUniqueID: <strong>'.$_SESSION['simplesamlphp_otrs_user_id'].'</strong></li>
                        			<li>sn: <strong>'.$_SESSION['simplesamlphp_otrs_user_sn'].'</strong></li>
                        			<li>givenName: <strong>'.$_SESSION['simplesamlphp_otrs_user_givenname'].'</strong></li>
                        			<li>mail: <strong>'.$_SESSION['simplesamlphp_otrs_user_mail'].'</strong></li>
									<li>hrEduPersonHomeOrg: <strong>'.$attributes['hrEduPersonHomeOrg'][0].'</strong></li>
                        		</ul>
				</td>
			</tr>';
	}
	/* Realms are not OK */
	else {
		echo '
			<tr height="60">
				<td align="left" valign="middle">
				Sorry, but it seems that you are comming from IdP that is not allowed to use this system.
				</td>
			</tr>
			<tr height="60">
				<td align="left" valign="middle">
				Allowed IdP realms are: <strong>'. ALLOWED_REALMS .'</strong>	
				</td>
			</tr>
		';
	}
	echo '
			<tr height="30">
				<td class="footer" align="center" colspan="3">SAML Single Sign-On for OTRS.</td>
			</tr>
		</table>
		</p>
                </body>
                </html>';
	session_destroy();
	exit();
}


/* Store SSO session data into a file: */
$sessionfile = fopen( "/var/run/samlsessions/sess_".session_id(), "w" );
fputs( $sessionfile, session_encode() );
fclose( $sessionfile );

/* Verify in OTRS database if user's account is disabled: */
$db = new DB();
$sql = "SELECT * FROM customer_user WHERE login='".$_SESSION['simplesamlphp_otrs_user_id']."'";
$db->query( $sql );

if( $db->num_rows() ){
	$db->next_record();
	if( $db->f( "valid_id" ) == 1 ){
        	/* If user's customer account hasn't been disabled, redirect him/her 
		to the OTRS customer interface (this URL should point to the location 
		of .../otrs/customer.pl script): */
        	header( 'location: '.OTRS_BASE_URL.'/customer.pl' );
	}
	else{
        	/* Otherwise, display information that user does not have customer privileges: */
		echo ' 	<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">
                	<html>
                	<head>
                        	<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                        	<title>OTRS</title>
				<style type="text/css">
				<!--
				.header { color:#efefef;background-color:#ffffff;font-weight:bold;font-size:16px;font-family:Geneva,Helvetica,Arial,sans-serif;border-style: none none solid none; border-color:#bcd5e4;}
				.footer { color:#193153;background-color:#bcd5e4;font-weight:bold;font-size:10px;font-family:Geneva,Helvetica,Arial,sans-serif; }
				-->
				</style>
                	</head>
			<body>
			<table width="850" height="400" border="0">
				<tr height="60">
					<td class="header">&nbsp;</td>
				</tr>
				<tr height="340">
					<td align="center" valign="middle" colspan="2">Sorry, but it seems that OTRS admin has disabled your customer account.</td>
				</tr>
				<tr height="30">
					<td class="footer" align="center" colspan="3">SAML Single Sign-On for OTRS.</td>
				</tr>
			</table>
			</body>
        		</html>';
	}
}
else{
	/* Allow user to login for the first time... */
       	header( 'location: '.OTRS_BASE_URL.'/customer.pl' );
}

?>
